'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MoleculerLink = undefined;

var _apolloLink = require('apollo-link');

var _printer = require('graphql/language/printer');

function createMoleculerLink(opts) {
  return new _apolloLink.ApolloLink(operation => new _apolloLink.Observable(observer => {
    const { credentials, fetcherOptions, graphqlContext } = operation.getContext();
    const { operationName, extensions, variables, query } = operation;
    const { broker, service } = opts;

    broker.call(`${service}.graphql`, {
      credentials,
      query: (0, _printer.print)(query),
      variables,
      extensions,
      operationName,
      graphqlContext
    }).then(result => {
      observer.next(result);
      observer.complete();
      return result;
    }).catch(err => {
      observer.error(err);
    });
  }));
}

class MoleculerLink extends _apolloLink.ApolloLink {

  constructor(opts) {
    super();
    this.requester = createMoleculerLink(opts).request;
  }

  request(op) {
    return this.requester(op);
  }
}
exports.MoleculerLink = MoleculerLink;