'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildRelationalResolvers = buildRelationalResolvers;

var _selectn = require('selectn');

var _selectn2 = _interopRequireDefault(_selectn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getArgs = (relationship, props) => {
  if (!relationship.args) return {};
  const computedArgs = {};
  for (const arg of Object.keys(relationship.args)) {
    computedArgs[arg] = (0, _selectn2.default)(relationship.args[arg], props);
  }
  return computedArgs;
};

const getFieldResolvers = (relationships, mergeInfo) => {
  const relationshipResolvers = {};
  for (const fieldName of Object.keys(relationships)) {
    const definition = relationships[fieldName];
    relationshipResolvers[fieldName] = {
      resolve(parent, args, context, info) {
        return mergeInfo.delegate(definition.type, definition.operationName, getArgs(definition, { parent, args, context, info }), context, info);
      }
    };
  }
  return relationshipResolvers;
};

function buildRelationalResolvers(typeDefinitions, validatedRelationalTypes) {
  return mergeInfo => {
    const typeResolvers = {};
    for (const type of Object.keys(typeDefinitions)) {
      if (validatedRelationalTypes && !validatedRelationalTypes.includes(type)) continue;
      typeResolvers[type] = getFieldResolvers(typeDefinitions[type], mergeInfo);
    }

    return typeResolvers;
  };
}