'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphQLGateway = undefined;

var _graphqlTools = require('graphql-tools');

var _graphql = require('graphql');

var _lodash = require('lodash.difference');

var _lodash2 = _interopRequireDefault(_lodash);

var _selectn = require('selectn');

var _selectn2 = _interopRequireDefault(_selectn);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _createRemoteSchema = require('./createRemoteSchema');

var _buildRelationalResolvers = require('./buildRelationalResolvers');

var _utilities = require('./utilities');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class GraphQLGateway {

  constructor(opts) {
    var _this = this;

    this.blacklist = ['$node'];
    this.discoveredTypes = {};
    this.expectedTypes = [];
    this.generateSnapshot = false;
    this.initialized = false;
    this.relationDefinitions = {};
    this.relationships = {};
    this.remoteSchemas = {};
    this.schema = null;
    this.service = null;
    this.snapshotPath = `${process.cwd()}/schema.snapshot.graphql`;
    this.waitTimeout = 5000;
    this.waitInterval = 100;

    this.handleServiceUpdate = (() => {
      var _ref = _asyncToGenerator(function* (opts) {
        const services = _this.broker.services.filter(function (service) {
          return service.settings.hasGraphQLSchema;
        }).filter(function (service) {
          return !_this.blacklist.includes(service.name);
        }).filter(function (service) {
          return !_this.discoveredTypes[service.settings.typeName];
        });

        if (services.length > 0) {
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = services[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              const service = _step.value;

              _this.discoveredTypes[service.settings.typeName] = service.name;
              yield _this.buildRemoteSchema(service);
              if (_this.onServiceDiscovery) {
                _this.onServiceDiscovery(service);
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          _this.generateSchema();
        }
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })();

    this.handleNodeConnection = (() => {
      var _ref2 = _asyncToGenerator(function* ({ node }) {
        const services = node.services.filter(function (service) {
          return !_this.discoveredTypes[service.settings.typeName] && !_this.blacklist.includes(service.name) && service.settings.hasGraphQLSchema;
        });
        if (services.length > 0) {
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = services[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              const service = _step2.value;

              _this.discoveredTypes[service.settings.typeName] = service.name;
              yield _this.buildRemoteSchema(service);
              if (_this.onServiceDiscovery) {
                _this.onServiceDiscovery(service);
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }

          _this.generateSchema();
        }
      });

      return function (_x2) {
        return _ref2.apply(this, arguments);
      };
    })();

    this.handleNodeDisconnected = (() => {
      var _ref3 = _asyncToGenerator(function* ({ node }) {
        const services = node.services.filter(function (service) {
          return _this.remoteSchemas[service.settings.typeName];
        });
        if (services.length > 0) {
          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = services[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              const service = _step3.value;

              yield _this.buildRemoteSchema(service);
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }

          _this.generateSchema();
        }
      });

      return function (_x3) {
        return _ref3.apply(this, arguments);
      };
    })();

    this.broker = opts.broker;
    if (opts.expectedTypes) this.expectedTypes = opts.expectedTypes;
    if (opts.waitInterval) this.waitInterval = opts.waitInterval;
    if (opts.waitTimeout) this.waitTimeout = opts.waitTimeout;
    if (opts.blacklist) this.blacklist.concat(opts.blacklist);
    if (opts.generateSnapshot) this.generateSnapshot = opts.generateSnapshot;
    if (opts.snapshotPath) this.snapshotPath = opts.snapshotPath;
    if (opts.onServiceDiscovery) this.onServiceDiscovery = opts.onServiceDiscovery;
    this.service = this.broker.createService({
      name: 'gateway',
      events: {
        '$services.changed': this.handleServiceUpdate,
        '$node.connected': this.handleNodeConnection,
        '$node.disconnected': this.handleNodeDisconnected
      }
    });
  }

  alphabetizeSchema(schema) {
    const queryType = schema._queryType;
    const fields = queryType.getFields();
    const unordered = Object.keys(fields);
    const ordered = Object.keys(fields).sort();
    if (JSON.stringify(unordered) !== JSON.stringify(ordered)) {
      const alphabetized = {};
      ordered.forEach(field => {
        alphabetized[field] = fields[field];
      });
      queryType._fields = alphabetized;
      schema._queryType = queryType;
    }
    return schema;
  }

  buildRemoteSchema(service) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      var _service$settings = service.settings;
      const typeName = _service$settings.typeName,
            relationships = _service$settings.relationships,
            relationDefinitions = _service$settings.relationDefinitions;

      if (!_this2.remoteSchemas[typeName]) {
        _this2.remoteSchemas[typeName] = yield (0, _createRemoteSchema.createRemoteSchema)({
          broker: _this2.broker,
          service
        });
        if (relationships) {
          _this2.relationships[typeName] = relationships;
          _this2.relationDefinitions[typeName] = relationDefinitions;
          const relatedTypes = (0, _utilities.getRelatedTypes)((0, _graphql.parse)(relationships));
          const missingTypes = (0, _lodash2.default)(relatedTypes, _this2.discoveredTypes);
          _this2.expectedTypes = _this2.expectedTypes.concat(missingTypes);
        }
      }
    })();
  }

  generateSchema() {
    const schemas = Object.values(this.remoteSchemas).concat(Object.values(this.relationships));
    const resolvers = (0, _buildRelationalResolvers.buildRelationalResolvers)(this.relationDefinitions);
    this.schema = (0, _graphqlTools.mergeSchemas)({
      schemas,
      resolvers
    });
    this.schema = this.alphabetizeSchema(this.schema);
    if (this.generateSnapshot) this.recordSnapshot();
    return this.schema;
  }

  recordSnapshot() {
    if (this.schema) {
      _fs2.default.writeFileSync(this.snapshotPath, (0, _graphql.printSchema)(this.schema));
    }
  }

  start() {
    return new Promise((resolve, reject) => {
      const maxTries = this.waitTimeout / this.waitInterval;
      let tries = 0;
      this.timer = setInterval(() => {
        tries++;
        if (tries >= maxTries) {
          reject(new Error('Timeout'));
        }
        const discoveredTypes = Object.keys(this.discoveredTypes);
        const undiscovered = (0, _lodash2.default)(this.expectedTypes, discoveredTypes);
        if (discoveredTypes.length === 0) return;
        if (discoveredTypes.some(type => !this.remoteSchemas[type])) return;
        if (undiscovered.length > 0) {
          if (this.broker.logger) {
            const msg = `Still waiting for ${undiscovered.join(', ')} types to be discovered`;
            this.broker.logger.warn(msg);
          }
          return;
        }
        clearInterval(this.timer);
        this.generateSchema();
        resolve(this.schema);
      }, this.waitInterval);
    });
  }
}
exports.GraphQLGateway = GraphQLGateway;