'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MoleculerLink = undefined;

var _apolloLink = require('apollo-link');

var _printer = require('graphql/language/printer');

function createMoleculerLink(opts) {
  return new _apolloLink.ApolloLink(operation => new _apolloLink.Observable(observer => {
    var _operation$getContext = operation.getContext();

    const credentials = _operation$getContext.credentials,
          fetcherOptions = _operation$getContext.fetcherOptions,
          graphqlContext = _operation$getContext.graphqlContext;
    const operationName = operation.operationName,
          extensions = operation.extensions,
          variables = operation.variables,
          query = operation.query;
    const broker = opts.broker,
          service = opts.service;


    broker.call(`${service}.graphql`, {
      credentials,
      query: (0, _printer.print)(query),
      variables,
      extensions,
      operationName,
      graphqlContext
    }).then(result => {
      observer.next(result);
      observer.complete();
      return result;
    }).catch(err => {
      observer.error(err);
    });
  }));
}

class MoleculerLink extends _apolloLink.ApolloLink {

  constructor(opts) {
    super();
    this.requester = createMoleculerLink(opts).request;
  }

  request(op) {
    return this.requester(op);
  }
}
exports.MoleculerLink = MoleculerLink;