'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildRelationalResolvers = buildRelationalResolvers;

var _selectn = require('selectn');

var _selectn2 = _interopRequireDefault(_selectn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getArgs = (relationship, props) => {
  if (!relationship.args) return {};
  const computedArgs = {};
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(relationship.args)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      const arg = _step.value;

      computedArgs[arg] = (0, _selectn2.default)(relationship.args[arg], props);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return computedArgs;
};

const getFieldResolvers = (relationships, mergeInfo) => {
  const relationshipResolvers = {};
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = Object.keys(relationships)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      const fieldName = _step2.value;

      const definition = relationships[fieldName];
      relationshipResolvers[fieldName] = {
        resolve(parent, args, context, info) {
          return mergeInfo.delegate(definition.type, definition.operationName, getArgs(definition, { parent, args, context, info }), context, info);
        }
      };
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return relationshipResolvers;
};

function buildRelationalResolvers(typeDefinitions, validatedRelationalTypes) {
  return mergeInfo => {
    const typeResolvers = {};
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = Object.keys(typeDefinitions)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        const type = _step3.value;

        if (validatedRelationalTypes && !validatedRelationalTypes.includes(type)) continue;
        typeResolvers[type] = getFieldResolvers(typeDefinitions[type], mergeInfo);
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }

    return typeResolvers;
  };
}