'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createGraphqlMixin = undefined;

var _graphqlTools = require('graphql-tools');

var _graphql = require('graphql');

const createGraphqlMixin = exports.createGraphqlMixin = ({
  typeName,
  schema,
  resolvers,
  relationships,
  relationDefinitions,
  directiveResolvers
}) => ({
  settings: {
    typeName,
    schema,
    relationships,
    relationDefinitions,
    directiveResolvers,
    hasGraphQLSchema: true
  },
  actions: {
    graphql: {
      params: {
        query: { type: 'string' },
        variables: { type: 'object', optional: true }
      },
      handler(ctx) {
        return (0, _graphql.graphql)(this.schema, ctx.params.query, this.resolvers, ctx, ctx.params.variables);
      }
    }
  },
  created() {
    this.resolvers = resolvers;
    this.schema = (0, _graphqlTools.makeExecutableSchema)({ typeDefs: [schema], resolvers, directiveResolvers });
  },
  started() {
    this.broker.broadcast('graphqlService.connected', {
      typeName,
      schema,
      relationships,
      relationDefinitions,
      directiveResolvers
    });
  },
  stopped() {
    this.broker.broadcast('graphqlService.disconnected', { typeName });
  }
});